//
//  CameraViewController.swift
//  SwiftImplementation
//
//  Created by Alex S. on 05.04.2018.
//  Copyright © 2018 Abycus. All rights reserved.
//

import BarcodeScanner

extension String: NotificationStrings {
	static var DecoderResultNotification: String {
		return "DecoderResultNotification"
	}
}

protocol NotificationStrings {
	static var DecoderResultNotification : String {get}
}

extension Notification.Name {
	static let DecoderResultNotification = Notification.Name(rawValue: .DecoderResultNotification)
}

let MAX_THREADS : Int = 2

import UIKit

class CameraViewController: UIViewController, AVCaptureVideoDataOutputSampleBufferDelegate, UIAlertViewDelegate {

	var state : MainScreenState!

	var captureSession : AVCaptureSession!
	var prevLayer: AVCaptureVideoPreviewLayer!
	var device : AVCaptureDevice!
	var focusTimer: Timer!

	var zoomLevel1 = 0
	var zoomLevel2 = 0
	var zoomLevel = 0

	var firstZoom : Float?
	var secondZoom : Float?

	var activeThreads : Int!
	var videoZoomSupported : Bool!

	//MARK: - lifecircle

    override func viewDidLoad() {
        super.viewDidLoad()

		NotificationCenter.default.addObserver(self, selector: #selector(decodeResultNotification), name: .DecoderResultNotification, object: nil)
        // Do any additional setup after loading the view.
    }

	override func viewWillAppear(_ animated: Bool) {
		super.viewWillAppear(animated)

		#if TARGET_IPHONE_SIMULATOR
			print("On iOS simulator camera is not Supported")
		#else
			self.initCapture()
		#endif
		self.startScanning()
	}

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

	override func viewWillDisappear(_ animated: Bool) {
		super.viewWillDisappear(animated)
		self.stopScanning()
		self.deinitCapture()
	}

	func initCapture() {

		let scanner = Barcode2DScanner()
		scanner.registerCode(UserDefaults.standard.object(forKey: "cameraKey") as! String)

		self.device = AVCaptureDevice.default(for: AVMediaType.video)

		var captureInput : AVCaptureDeviceInput? = nil
		do {
			captureInput = try AVCaptureDeviceInput.init(device: self.device)
		} catch  {}

		if (captureInput == nil) {
			let appName = Bundle.main.infoDictionary![kCFBundleNameKey! as String]

			let alertController = UIAlertController(title: "Camera Unavaliable", message: "The \(appName) has not been given a permission to your camera. Please check the Privacy Settings: Settings -> \(appName) -> Privacy -> Camera", preferredStyle: .alert)
			let okAction = UIAlertAction(title: "OK", style: .default, handler: nil)
			return
		}

		var captureOutput = AVCaptureVideoDataOutput()
		captureOutput.setSampleBufferDelegate(self as! AVCaptureVideoDataOutputSampleBufferDelegate, queue: DispatchQueue.main)


		self.captureSession = AVCaptureSession()
		self.captureSession.addInput(captureInput!)
		self.captureSession.addOutput(captureOutput)

		var hostInfo : host_basic_info
		var infoCount : mach_msg_type_number_t
		infoCount = mach_msg_type_number_t(HOST_BASIC_INFO)

		host_info(mach_host_self(), HOST_BASIC_INFO, &hostInfo, infoCount)
		if (self.captureSession.canSetSessionPreset(.high)) {
			print("Set previre port to 1920x1080")
			self.captureSession.sessionPreset = .high
		} else {
			if (self.captureSession.canSetSessionPreset(.hd1280x720)) {
				print("Set previre port ro 1080x720")
				self.captureSession.sessionPreset = .hd1280x720
			}
		}

		host_info(mach_host_self(), HOST_BASIC_INFO, &hostInfo, infoCount)

		if (hostInfo.max_cpus < 2) {
			if (self.device.responds(to: #selector(getter: AVCaptureDevice.activeVideoMinFrameDuration))) {

				do {
					try self.device.lockForConfiguration()
				} catch {
					print("Exception: \(error)")
				}
				self.device.activeVideoMinFrameDuration = CMTime(value: 1, timescale: 15)
				self.device.unlockForConfiguration()

			} else {
				//var conn : AVCaptureConnection = captureOutput.connection(with: .video)!
				self.device.activeVideoMinFrameDuration = CMTimeMake(1, 15)
			}
		}

		print("hostInfo.max_cpus \(hostInfo.max_cpus)")
		var avaliableThreads = min(MAX_THREADS, hostInfo.max_cpus as!  Int)
		self.activeThreads = 0

		//prev layer

		self.prevLayer = AVCaptureVideoPreviewLayer.init(session: self.captureSession)


		switch (self.interfaceOrientation) {
		case .landscapeLeft:
			self.prevLayer.connection?.videoOrientation = .landscapeLeft
			self.prevLayer.frame = CGRect(x: 0, y: 0, width: max(self.view.frame.size.width, self.view.frame.size.height), height: min(self.view.frame.size.width, self.view.frame.size.height))
			break
		case .landscapeRight:
			self.prevLayer.connection?.videoOrientation = .landscapeRight;
			self.prevLayer.frame = CGRect(x:0, y:0, width: max(self.view.frame.size.width,self.view.frame.size.height), height: min(self.view.frame.size.width,self.view.frame.size.height))
			break
		case .portrait:
			self.prevLayer.connection?.videoOrientation = .portrait;
			self.prevLayer.frame = CGRect(x: 0, y: 0, width: min(self.view.frame.size.width,self.view.frame.size.height), height: max(self.view.frame.size.width,self.view.frame.size.height))
			break
		case .portraitUpsideDown:
			self.prevLayer.connection?.videoOrientation = .portraitUpsideDown;
			self.prevLayer.frame = CGRect(x: 0, y: 0, width: min(self.view.frame.size.width,self.view.frame.size.height), height: max(self.view.frame.size.width,self.view.frame.size.height))
		default: break
		}

		self.prevLayer.videoGravity = .resizeAspectFill
		self.view.layer.addSublayer(self.prevLayer)
		#if USE_MWOVERLAY
			MWOverlay.addToPreviewLayer(self.prevLayer)
		#endif

		self.videoZoomSupported = false

		if (self.device.responds(to: #selector(getter: AVCaptureDevice.activeFormat)) &&
			self.device.activeFormat.responds(to: Selector("videoMaxZoomFactor")) &&
			self.device.responds(to: #selector(getter: AVCaptureDevice.videoZoomFactor))) {

			var maxZoom : Float = 0
			if (self.device.activeFormat.responds(to: Selector("videoZoomFactorUpscaleThreshold"))) {
				maxZoom = Float((self.device?.activeFormat.videoZoomFactorUpscaleThreshold)!)
			} else {
				maxZoom = Float((self.device?.activeFormat.videoMaxZoomFactor)!)
			}
			var maxZoomTotal = Int((self.device?.activeFormat.videoMaxZoomFactor)!)

			if (self.device.responds(to: #selector(getter: AVCaptureDevice.videoZoomFactor)) && (Float(maxZoomTotal) > 1.1)) {

				var videoZoomSupported = true;

				if(zoomLevel1 != 0 && zoomLevel2 != 0) {

					if (zoomLevel1 > maxZoomTotal*100) {zoomLevel1 = maxZoomTotal*100 as Int}
					if (zoomLevel2 > maxZoomTotal*100) {zoomLevel2 = maxZoomTotal*100 as Int}
					firstZoom = Float(zoomLevel1) * 0.01
					secondZoom = Float(zoomLevel2) * 0.01

				} else {

					if (maxZoomTotal > 2) {

						if (maxZoom > 1.0 && maxZoom <= 2.0) {
							firstZoom = maxZoom
							secondZoom = maxZoom * 2
						} else {
							if (maxZoom > 2.0) {
								firstZoom = 2.0
								secondZoom = 4.0
							}
						}
					}
				}
			}
		}

		self.focusTimer = Timer.init(timeInterval: 1.0, target: self, selector: #selector(reFocus), userInfo: nil, repeats: true)
	}

	//MARK: - Methods

	@objc func reFocus() {
		var error: Error
		do {
			try self.device?.lockForConfiguration()
			if (self.device.isFocusPointOfInterestSupported) {
				self.device.focusPointOfInterest = CGPoint(x: 0.49, y: 0.49)
				self.device.focusMode = .autoFocus
			}
			self.device.unlockForConfiguration()

		} catch  {

		}

	}

	func startScanning() {

	}

	func stopScanning() {

	}

	func deinitCapture() {

	}

	@objc func decodeResultNotification() {

	}


}
